"use strict";

var _createReducer = require("./createReducer");

var _createReducer2 = _interopRequireDefault(_createReducer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var testReducerDispatchAction = function testReducerDispatchAction(_ref) {
  var reducer = _ref.reducer,
      action = _ref.action,
      expectState = _ref.expectState;

  describe("when action '" + action.type + "' has payload '" + action.payload.toString() + "'  ", function () {
    var state = void 0;

    beforeAll(function () {
      state = reducer(undefined, action);
    });

    it("then state should be '" + expectState.toString() + "'", function () {
      expect(state).toBe(expectState);
    });
  });
};

describe("create reducer", function () {
  describe("given simple reducer", function () {
    var reducer = (0, _createReducer2.default)(false, {
      A: function A(state, payload) {
        return payload;
      }
    });

    testReducerDispatchAction({
      reducer: reducer,
      action: { type: "A", payload: true },
      expectState: true
    });

    testReducerDispatchAction({
      reducer: reducer,
      action: { type: "B", payload: true },
      expectState: false
    });
  });

  describe("given nested reducer", function () {
    var reducer = (0, _createReducer2.default)(false, {
      A: {
        B: {
          C1: function C1(state, payload) {
            return payload;
          },
          C2: {
            D: function D(state, payload) {
              return payload;
            }
          }
        }
      }
    });

    testReducerDispatchAction({
      reducer: reducer,
      action: { type: "A", payload: true },
      expectState: false
    });

    testReducerDispatchAction({
      reducer: reducer,
      action: { type: "A_B", payload: true },
      expectState: false
    });

    testReducerDispatchAction({
      reducer: reducer,
      action: { type: "A_B_C1", payload: true },
      expectState: true
    });

    testReducerDispatchAction({
      reducer: reducer,
      action: { type: "A_B_C2", payload: true },
      expectState: false
    });

    testReducerDispatchAction({
      reducer: reducer,
      action: { type: "A_B_C2_D", payload: true },
      expectState: true
    });
  });

  describe("given incorrect reducer", function () {
    it("should throw error", function () {
      expect(function () {
        return (0, _createReducer2.default)(false, {
          A: false
        });
      }).toThrow();
    });
  });
});